﻿ <#
	This script will run the function Get-UserInfoHTML and bring the CSV or HTML output from your AD
	
	Parameter description:
		This script will receive a parameter called "Selector" 
		it accepts 3 options: HTML (default), CSV or BOTH
		if you run the script like:
			- .\GetUsersInfoHTML.ps1 -Selector HTML => Same as default option and you will get the HTML5 report with all the properties of all users in your AD environment
			- .\GetUsersInfoHTML.ps1 -Selector CSV  => You will get a CSV file with all the properties of all users in your AD environment, ready to import into excel or use it as feed to another powershell script (custom actions, even email send)
			- .\GetUsersInfoHTML.ps1 -Selector BOTH => Will run both reports HTML and CSV.
	how to use this script?
	
	* Get simple HTML5 report
		.\GetUsersInfoHTML.ps1
		.\GetUsersInfoHTML.ps1 -Selector HTML 
		 
	* Get Simple CSV file 
		.\GetUsersInfoHTML.ps1 -Selector CSV 
		
	* Get CSV and HTML5 report
		.\GetUsersInfoHTML.ps1 -Selector BOTH 
	
    Even you can see the date of the last bad logon attempt of each users. That's the power of powershell.
#>
[CmdletBinding()]
param(
      [Parameter(ValueFromPipeline=$true,Position=1,Mandatory=$false)][ValidateSet("HTML","CSV","BOTH")]$Selector="HTML"
)
function Write-Log{
    [CmdletBinding()]
    #[Alias('wl')]
    [OutputType([int])]
    Param(
            # The string to be written to the log.
            [Parameter(Mandatory=$true, ValueFromPipelineByPropertyName=$true, Position=0)] [ValidateNotNullOrEmpty()] [Alias("LogContent")] [string]$Message,
            # The path to the log file.
            [Parameter(Mandatory=$false, ValueFromPipelineByPropertyName=$true,Position=1)] [Alias('LogPath')] [string]$Path=$global:DefaultLog,
            [Parameter(Mandatory=$false, ValueFromPipelineByPropertyName=$true,Position=2)] [ValidateSet("Error","Warn","Info","Load","Execute")] [string]$Level="Info",
            [Parameter(Mandatory=$false)] [switch]$NoClobber
    )

     Process{
        
        if ((Test-Path $Path) -AND $NoClobber) {
            Write-Warning "Log file $Path already exists, and you specified NoClobber. Either delete the file or specify a different name."
            Return
            }

        # If attempting to write to a log file in a folder/path that doesn't exist
        # to create the file include path.
        elseif (!(Test-Path $Path)) {
            Write-Verbose "Creating $Path."
            $NewLogFile = New-Item $Path -Force -ItemType File
            }

        else {
            # Nothing to see here yet.
            }

        # Now do the logging and additional output based on $Level
        switch ($Level) {
            'Error' {
                Write-Warning $Message
                Write-Output "$(Get-Date -Format "yyyy-MM-dd HH:mm:ss") ERROR: `t $Message" | Out-File -FilePath $Path -Append
                }
            'Warn' {
                Write-Warning $Message
                Write-Output "$(Get-Date -Format "yyyy-MM-dd HH:mm:ss") WARNING: `t $Message" | Out-File -FilePath $Path -Append
                }
            'Info' {
                Write-Host $Message -ForegroundColor Green
                Write-Verbose $Message
                Write-Output "$(Get-Date -Format "yyyy-MM-dd HH:mm:ss") INFO: `t $Message" | Out-File -FilePath $Path -Append
                }
            'Load' {
                Write-Host $Message -ForegroundColor Magenta
                Write-Verbose $Message
                Write-Output "$(Get-Date -Format "yyyy-MM-dd HH:mm:ss") LOAD: `t $Message" | Out-File -FilePath $Path -Append
                }
            'Execute' {
                Write-Host $Message -ForegroundColor Green
                Write-Verbose $Message
                Write-Output "$(Get-Date -Format "yyyy-MM-dd HH:mm:ss") EXEC: `t $Message" | Out-File -FilePath $Path -Append
                }
            }
    }
}

function Get-UserInfoHTML{
    [CmdletBinding()]

    param(
        [Parameter(ValueFromPipeline=$true,Position=1,Mandatory=$false)][ValidateSet("HTML","CSV","BOTH")]$Selector="HTML"
        )

 BEGIN{
    Write-Verbose "Importing Active Directory neccesary CMDLETS"
    import-module activedirectory -CMDLet Get-ADUser,SetADUser
    $TodayDate=get-date -UFormat "%Y%m%d" #"%A-%Y%m%d"
    $dia = get-date -UFormat "%A"
    $fecha = $TodayDate
    $hora= Get-date -UFormat "%H:%M:%S"
    $FnHTML= "UsersInfo-$TodayDate.html"
    $FnCSV = "UsersInfo-$TodayDate.csv"

    $header= @"
    <style type=""text/css"">
    body {font-family: Verdana, Geneva, Arial, Helvetica, sans-serif;}
    #report { width: 835px; }
    table{border-collapse: collapse;border: none;
        font: 10pt Verdana, Geneva, Arial, Helvetica, sans-serif;
        color: black; margin-bottom: 10px;}
    table td{ font-size: 12px; padding-left: 0px; padding-right: 20px; text-align: left;}
    table th{ font-size: 12px; font-weight: bold; padding-left: 0px; padding-right: 20px; text-align: left;}
    h2{ clear: both; font-size: 130%;color:#354B5E; }
    h3{ clear: both; font-size: 75%; margin-left: 20px; margin-top: 30px; color:#475F77; }
    p{ margin-left: 20px; font-size: 12px; }
    table.list{ float: left; }
	table.list td:nth-child(1){font-weight: bold;border-right: 1px grey solid;text-align: right;}
    table.list td:nth-child(2){ padding-left: 7px; }
    table tr:nth-child(even) td:nth-child(even){ background: #BBBBBB; }
    table tr:nth-child(odd) td:nth-child(odd){ background: #F2F2F2; }
    table tr:nth-child(even) td:nth-child(odd){ background: #DDDDDD; }
    table tr:nth-child(odd) td:nth-child(even){ background: #E5E5E5; }
	div.column { width: 320px; float: left; }
    div.first{ padding-right: 20px; border-right: 1px grey solid; }
    div.second{ margin-left: 30px; }
    table{ margin-left: 20px; }
	</style>
"@;
    write-verbose "Getting Script Directory path"
 }
 PROCESS{
    Write-Verbose "Getting information from users"  
    $usersnames= get-aduser -SearchBase "OU=Example,DC=DOMAIN,DC=local" -SearchScope Subtree -filter 'enabled -eq "true"' -Properties *
	$computer = $env:computername
    $AllInfoProperties=@()
    foreach($user in $usersnames){
        $IDName=$user.SamAccountName
        $AllInfoProperties+=Get-ADUser -Identity $IDName -Properties * | Select *
    }
		$AllUsersFilter=  $AllInfoProperties| Sort-Object Displayname,Enabled | Select DisplayName,GivenName,Surname,EmailAddress,Description,HomePhone,MobilePhone,OfficePhone,PostalCode,Country,State,City,StreetAddress,Office,Organization,Company,Department,Title,Manager
		#Html5 part
     $html= '<!DOCTYPE HTML>
<html lang="en-US">
<head>
	<meta charset="UTF-8">
	<title>'
    $html+=$title
    $html+="</title>
	<style type=""text/css"">{margin:0;padding:0}@import url(https://fonts.googleapis.com/css?family=Indie+Flower|Josefin+Sans|Orbitron:500|Yrsa);body{text-align:center;font-family:14px/1.4 'Indie Flower','Josefin Sans',Orbitron,sans-serif;font-family:'Indie Flower',cursive;font-family:'Josefin Sans',sans-serif;font-family:Orbitron,sans-serif}#page-wrap{margin:50px}tr:nth-of-type(odd){background:#eee}th{background:#EF5525;color:#fff;font-family:Orbitron,sans-serif}td,th{padding:6px;border:1px solid #ccc;text-align:center;font-size:large}table{width:90%;border-collapse:collapse;margin-left:auto;margin-right:auto;font-family:Yrsa,serif}</style>
</head>
<body>
<img id=""logo"" src=""https://www.example.de/logo.svg""/>

<h2>System Information</h2>

<br>    
    <a href=""https://www.example.de"" target=""_blank"">Website</a>
<br>

<br>
<h2>User Information </h2>
<table>
<tr>
    <th>DisplayName</th><th>GivenName</th><th>Surname</th><th>EmailAddress</th><th>Description</th><th>HomePhone</th><th>MobilePhone</th><th>OfficePhone</th><th>PostalCode</th><th>Country</th><th>State</th><th>City</th><th>StreetAddress</th><th>Office</th><th>Organization</th><th>Company</th><th>Department</th><th>Title</th><th>Manager</th>
</tr>
"

foreach($item in  $AllUsersFilter){
   $html+="<tr> <td>$($item.DisplayName)</td><td>$($item.GivenName)</td><td>$($item.Surname)</td><td>$($item.EmailAddress)</td><td>$($item.Description)</td><td>$($item.HomePhone)</td> <td>$($item.MobilePhone)</td><td>$($item.OfficePhone)</td><td>$($item.PostalCode)</td><td>$($item.Country)</td><td>$($item.State)</td><td>$($item.City)</td><td>$($item.StreetAddress)</td><td>$($item.Office)</td><td>$($item.Organization)</td><td>$($item.Company)</td><td>$($item.Department)</td><td>$($item.Title)</td><td>$($item.Manager)</td></tr></tr>"
}

$html+="
</table>
<br>
<footer>
	<a href=""https://www.example.de"" target=""_blank"">
	Example Company</a>  - &#9400; All rigths reserved.
	</footer>
</body>
</html>"
   
 }
 END{
    if($Selector -eq "HTML" -or $Selector -eq "BOTH"){
        $html | Out-File "$global:ScriptLocation\$FnHTML" 
    }
    if($Selector -eq "CSV" -or $Selector -eq "BOTH"){
        $AllUsersFilter | ConvertTo-Csv -NoTypeInformation | out-file "$global:ScriptLocation\$FnCSV"
    }
 }
 }

function ShowTimeMS{
  [CmdletBinding()]
  param(
    [Parameter(ValueFromPipeline=$True,position=0,mandatory=$true)]	[datetime]$timeStart,
	[Parameter(ValueFromPipeline=$True,position=1,mandatory=$true)]	[datetime]$timeEnd
  )
  BEGIN {
    
  }
  PROCESS {
		write-Verbose "Stamping time"
		write-Verbose  "initial time: $TimeStart"
		write-Verbose "End time: $TimeEnd"
		$diff=New-TimeSpan $TimeStart $TimeEnd
		Write-verbose "Timediff= $diff"
		$miliseconds = $diff.TotalMilliseconds
		Write-output " Total Time in miliseconds is: $miliseconds ms"
  }
  END{
    Write-Log -Level Execute -Message "Finished $($diff.TotalSeconds) seconds"
  }
}




$TimeStart=Get-Date
$today = get-date -format MM-dd-yyyy

#Script Start

#Clean Up VariableS
$CleanUpVar=@()
$CleanUpGlobal=@()

#Get start time
$TimeStart=Get-Date
$CleanUpVar+="TimeStart"

#GLOBALs 
$global:ScriptLocation = $(get-location).Path
$global:DefaultLog = "$global:ScriptLocation\InfoUsers.log"
$CleanUpGlobal+="ScriptLocation"
$CleanUpGlobal+="DefaultLog"


write-log -Level Info "******************* Script Started $TimeStart *******************"

Get-UserInfoHTML -Selector $Selector

#Get the end time 
$TimeEnd=Get-Date

#showtime
write-log -Level Info "******************* Script Finished $TimeEnd *******************"

ShowTimeMS $TimeStart $TimeEnd
